from subprocess import CompletedProcess

import ipih

from pih import A, PIHThread
from pih.tools import while_not_do, j
from pih.collections import RobocopyJobDescription, RobocopyJobItem, RobocopyJobStatus
from BackupService.const import ROBOCOPY as RBK, PATH
from typing import Callable


class BackupApi:
    def start_robocopy_job(self, value: RobocopyJobItem, force: bool = False) -> bool:
        def internal_start_robocopy_job(
            job_item: RobocopyJobItem, force: bool = False
        ) -> bool:
            def run_robocopy_job(job_item: RobocopyJobItem) -> None:
                job_name: Callable[[], str] = lambda: self.get_job_name(
                    name, source, destination
                )
                source: str = job_item.source
                destination: str = job_item.destination
                name: str = job_item.name
                job_path: str = self.get_job_path(name, source, destination)
                job_status: RobocopyJobStatus | None = None
                if not job_item.live:
                    job_status = self.get_job_status(name, source, destination)
                    job_status.active = True
                    self.save_job_status(job_status)
                    A.E.backup_robocopy_job_was_started(job_name(), job_status)
                process: CompletedProcess = A.EXC.execute(
                    A.EXC.create_command_for_psexec(
                        (RBK.NAME, RBK.JOB_PARAMETER_NAME + job_path),
                        job_item.host,
                        interactive=not job_item.live,
                        run_from_system_account=job_item.run_from_system_account,
                        run_with_elevetion=job_item.run_with_elevetion,
                    ),
                    show_output=True,
                    capture_output=False,
                )
                last_status: int = -1
                if job_item.live:
                    pid: int = process.returncode
                    job_status = self.get_job_status(name, source, destination)
                    job_status.live = True
                    job_status.active = True
                    job_status.pid = pid
                    self.save_job_status(job_status)
                    A.E.backup_robocopy_job_was_started(job_name(), job_status)
                    while_not_do(
                        lambda: not A.EXC.process_is_exists(pid, job_item.host),
                        sleep_time=5,
                    )
                else:
                    last_status = process.returncode
                job_status = self.get_job_status(name, source, destination)
                job_status.active = False
                job_status.pid = -1
                job_status.last_created = A.D.now_to_string(A.CT.ISO_DATETIME_FORMAT)
                job_status.last_status = last_status
                self.save_job_status(job_status)
                A.E.backup_robocopy_job_was_completed(job_name(), job_status)

            #
            job_status: RobocopyJobStatus = self.get_job_status(
                job_item.name, job_item.source, job_item.destination
            )
            if not job_status.active or force:
                job_status.active = True
                self.save_job_status(job_status)
                PIHThread(run_robocopy_job, args=(job_item,))
                return True
            return False

        return internal_start_robocopy_job(value, force)

    @staticmethod
    def get_job_list() -> list[RobocopyJobItem]:
        rjm: dict[str, dict[str, list[RobocopyJobDescription]]] = RBK.JOB_MAP
        result: list[RobocopyJobItem] = []
        for source in rjm:
            for destination in rjm[source]:
                for job_description in rjm[source][destination]:
                    job_item: RobocopyJobItem = A.D.fill_data_from_source(
                        RobocopyJobItem(), job_description
                    )
                    job_item.source = source
                    job_item.destination = destination
                    job_item.name = job_item.name.lower()
                    result.append(job_item)
        return result

    @staticmethod
    def get_job_name(name: str, source: str, destination: str) -> str:
        result: str = ""
        if not A.D.is_empty(name):
            result += name
            if not A.D.is_empty(source):
                result += f":{source}"
                if not A.D.is_empty(destination):
                    result += f"{A.CT_V.ARROW}{destination}"
        return result

    @staticmethod
    def get_job_status(name: str, source: str, destination: str) -> RobocopyJobStatus:
        return A.R_DS.value(
            A.D_F.BACKUP.job_status_name(name, source, destination), RobocopyJobStatus
        ).data or RobocopyJobStatus(name, source, destination)

    @staticmethod
    def save_job_status(value: RobocopyJobStatus) -> bool:
        return A.A_DS.value(
            value,
            A.D_F.BACKUP.job_status_name(value.name, value.source, value.destination),
        )

    @staticmethod
    def get_job_path(name: str, source: str, destination: str) -> str:
        return A.PTH.join(
            PATH.JOB_CONFIG.DIRECTORY, j((source, destination), "-"), name
        )



import os

import ipih
from pih import A
from pih.collections import Result, RobocopyJobStatus
from pih.tools import ParameterList
from BackupService.api import *
from BackupService.const import *
from datetime import datetime
from typing import Any
import grpc


import json

SC = A.CT_SC

ISOLATED: bool = False


class DH:
    job_config_file_path: str = ""


def start(as_standalone: bool = False) -> None:

    def load_job_config_file() -> None:
        DH.job_config_file_path = A.SE.named_arg(JOB_CONFIG_ALIAS)
        work_directory: str = os.path.dirname(os.path.realpath(__file__))

        job_config_file_default_path: str = A.PTH.join(
            work_directory, PATH.JOB_CONFIG.FILE_NAME
        )
        DH.job_config_file_path = (
            DH.job_config_file_path or job_config_file_default_path
        )
        if not A.PTH.exists(DH.job_config_file_path) and A.PTH.exists(
            job_config_file_default_path
        ):
            DH.job_config_file_path = job_config_file_default_path
            A.O.error("Config job file not found")
            A.O.blue(f"Used default config job file: {job_config_file_default_path}")
        if A.PTH.exists(DH.job_config_file_path):
            data: dict = json.load(open(DH.job_config_file_path))
            PATH.JOB_CONFIG.DIRECTORY_NAME = data["job_config_directory"]
            PATH.JOB_CONFIG.DIRECTORY = A.PTH.join(
                A.PTH.get_file_directory(DH.job_config_file_path),
                PATH.JOB_CONFIG.DIRECTORY_NAME,
            )
            ROBOCOPY.JOB_MAP = data["job_config_map"]
        else:
            A.O.error("Config job file not found")
            A.SE.exit(0)

    A.SE.add_isolated_arg()
    A.SE.add_arg(JOB_CONFIG_ALIAS, nargs="?", const="True", type=str)

    def robocopy_job_handler(
        name: str | None,
        source: str | None,
        destination: str | None,
        force: bool = False,
    ) -> bool:
        rjl: list[RobocopyJobItem] = BackupApi.get_job_list()
        rjl = (
            rjl
            if A.D_C.empty(name)
            else list(filter(lambda item: item.name == name, rjl))
        )
        rjl = (
            rjl
            if A.D_C.empty(source)
            else list(filter(lambda item: item.source == source, rjl))
        )
        rjl = (
            rjl
            if A.D_C.empty(destination)
            else list(filter(lambda item: item.destination == destination, rjl))
        )
        for job_item in rjl:
            BackupApi().start_robocopy_job(job_item, force)
        return len(rjl) > 0

    def heat_beat_action(current_datetime: datetime) -> None:
        if A.D.is_not_none(ROBOCOPY.JOB_MAP):
            rjl: list[RobocopyJobItem] = BackupApi.get_job_list()
            for rji in rjl:
                if A.D_C.by_secondless_time(current_datetime, rji.start_datetime):
                    BackupApi().start_robocopy_job(rji)

    def service_call_handler(sc: SC, pl: ParameterList, context) -> Any:
        if sc == SC.heart_beat:
            heat_beat_action(A.D_Ex.parameter_list(pl).get())
            return True
        if sc == SC.robocopy_start_job:
            if robocopy_job_handler(
                pl.next(),
                pl.next(),
                pl.next(),
                pl.next(),
            ):
                return True
            return A.ER.rpc(
                context,
                f"Robocopy job: '{pl.values[0: 3]}'",
                grpc.StatusCode.NOT_FOUND,
            )
        if sc == SC.robocopy_get_job_status_list:
            rjl: list[RobocopyJobItem] = BackupApi.get_job_list()
            result: list[RobocopyJobStatus] = []
            for job_item in rjl:
                result.append(
                    BackupApi.get_job_status(
                        job_item.name, job_item.source, job_item.destination
                    )
                )
            return Result(None, result)

    def service_starts_handler() -> None:
        A.SRV_A.subscribe_on(SC.heart_beat)
        A.L.services_bot(
            j(("Backup::", JOB_CONFIG_ALIAS, ": ", DH.job_config_file_path))
        )

    load_job_config_file()

    A.SRV_A.serve(
        SD,
        service_call_handler,
        service_starts_handler,
        isolate=ISOLATED,
        as_standalone=as_standalone,
    )


if __name__ == "__main__":
    start()



import ipih

from pih import A
from pih.collections import RobocopyJobDescription
from pih.collections.service import ServiceDescription

NAME: str = "Backup"

VERSION: str = "0.11"


JOB_CONFIG_ALIAS: str = "job_config"


class ROBOCOPY:

    NAME: str = "robocopy"
    JOB_PARAMETER_NAME: str = "/job:"

    JOB_MAP: dict[str, dict[str, list[RobocopyJobDescription]]] | None = None

    class JOB_NAMES:

        MOVE_1C_BACKUPS: str = "move_1c_backups"
        POLIBASE_DATA: str = "polibase_data"
        POLIBASE_DATA_LIVE: str = "polibase_data_live"
        POLIBASE_FILES: str = "polibase_files"
        OMS: str = "oms"
        SCAN: str = "scan"
        HOMES: str = "homes"
        SHARES: str = "shares"
        FACADE: str = "facade"
        USER_DESKTOP: str = "user_desktop"
        VALENTA: str = A.CT.VALENTA.NAME


HOST = A.CT_H.BACKUP_WORKER


class PATH:

    class JOB_CONFIG:

        FILE_NAME: str = A.PTH.add_extension(JOB_CONFIG_ALIAS, A.CT_F_E.JSON)
        DIRECTORY_NAME: str = "robocopy_config"
        DIRECTORY: str = A.PTH.join(
            A.PTH.FACADE.SERVICE(NAME),
            DIRECTORY_NAME,
        )

        VALUE: str = A.PTH.join(A.PTH.FACADE.SERVICE(NAME), FILE_NAME)


SD: ServiceDescription = ServiceDescription(
    name=NAME,
    description="Backup service",
    host=HOST.NAME,
    commands=("robocopy_start_job", "robocopy_get_job_status_list"),
    version=VERSION,
    standalone_name="bck",
    use_standalone=True,
    parameters={JOB_CONFIG_ALIAS: PATH.JOB_CONFIG.VALUE},
)



import ipih

from pih import A
from BackupService.const import *
import json


# \\pih\facade\WSService\pstools\psexec -nobanner -accepteula \\ws-255 -i -h -u fmv\Administrator -p Fortun@90 robocopy /job:\\pih\facade\BackupService\robocopy_config\dc1-dc2\shares
# \\pih\facade\WSService\pstools\psexec -nobanner -accepteula \\backup_worker -i -u fmv\Administrator -p Fortun@90 robocopy /job:\\pih\facade\BackupService\robocopy_config\1c-nas\move_1c_backups
# \\pih\facade\WSService\pstools\psexec -nobanner -accepteula \\ws-255 -i -h -u fmv\Administrator -p Fortun@90 robocopy /job:\\pih\facade\BackupService\robocopy_config\polibase-polibase_test\polibase_data

JOB_SETTINGS_DEFAULT: RobocopyJobDescription = RobocopyJobDescription(
    None, None, A.CT.HOST.WS255.NAME, False, True, False
)

JOB_NAMES = ROBOCOPY.JOB_NAMES

USER_DESKTOP_SETTINGS: dict[str, list[RobocopyJobDescription]] = {
    A.CT.HOST.DC1.ALIAS: [JOB_SETTINGS_DEFAULT.clone(JOB_NAMES.USER_DESKTOP)]
}

print(
    json.dumps(
        json.loads(
            A.D.rpc_represent(
                {
                    A.CT.HOST.DC1.ALIAS: {
                        A.CT.HOST.DC2.ALIAS: [
                            JOB_SETTINGS_DEFAULT.clone(JOB_NAMES.OMS),
                            JOB_SETTINGS_DEFAULT.clone(JOB_NAMES.SCAN),
                            JOB_SETTINGS_DEFAULT.clone(JOB_NAMES.SHARES),
                            JOB_SETTINGS_DEFAULT.clone(JOB_NAMES.HOMES),
                            JOB_SETTINGS_DEFAULT.clone(JOB_NAMES.FACADE),
                        ],
                        A.CT.HOST.BACKUP_WORKER.ALIAS: [
                            JOB_SETTINGS_DEFAULT.clone(JOB_NAMES.OMS),
                            JOB_SETTINGS_DEFAULT.clone(JOB_NAMES.SHARES),
                            JOB_SETTINGS_DEFAULT.clone(JOB_NAMES.HOMES),
                        ],
                    },
                    A.CT.HOST.POLIBASE.ALIAS: {
                        A.CT.HOST.BACKUP_WORKER.ALIAS: [
                            JOB_SETTINGS_DEFAULT.clone(JOB_NAMES.POLIBASE_DATA),
                            JOB_SETTINGS_DEFAULT.clone(
                                JOB_NAMES.POLIBASE_DATA_LIVE, live=True
                            ),
                            JOB_SETTINGS_DEFAULT.clone(JOB_NAMES.POLIBASE_FILES),
                        ],
                        A.CT.HOST.POLIBASE2.ALIAS: [
                            JOB_SETTINGS_DEFAULT.clone(JOB_NAMES.POLIBASE_DATA),
                            JOB_SETTINGS_DEFAULT.clone(
                                JOB_NAMES.POLIBASE_DATA_LIVE, live=True
                            ),
                            JOB_SETTINGS_DEFAULT.clone(JOB_NAMES.POLIBASE_FILES),
                        ],
                    },
                    A.CT.HOST._1C.ALIAS: {
                        A.CT.HOST.NAS.ALIAS: [
                            RobocopyJobDescription(JOB_NAMES.MOVE_1C_BACKUPS, "8:00")
                        ]
                    },
                    A.CT.HOST.WS816.NAME: {
                        A.CT.HOST.WS255.NAME: [
                            JOB_SETTINGS_DEFAULT.clone(JOB_NAMES.VALENTA)
                        ]
                    },
                    "gea": USER_DESKTOP_SETTINGS,
                    "ssa": USER_DESKTOP_SETTINGS,
                    "bar": USER_DESKTOP_SETTINGS,
                    "vns": USER_DESKTOP_SETTINGS,
                    "vvp": USER_DESKTOP_SETTINGS,
                    "gen": USER_DESKTOP_SETTINGS,
                    "rob": USER_DESKTOP_SETTINGS,
                    "ptyu": USER_DESKTOP_SETTINGS,
                }
            )
        ),
        indent=2,
    )
)
